      PROGRAM TIMEMARCH_LINBT
CCCCCCCCCCCCCCCCCCCCCCCCCc
CC  Model to get the steady forced solutions for the linearized barotropic
CC  vorticity equation.  Following Branstator, Krishnamurthi and Haltiner 
CC  and Williams

      IMPLICIT DOUBLE PRECISION (P,X)
      DOUBLE PRECISION PM,PD
      DIMENSION PM(0:100,0:100),PD(0:100,0:100)

C  For simplicity will assume that one degree is 110 kilometers.
      
      PARAMETER (DX=110000, DY=110000)
      PARAMETER (XGRIDS=144, YGRIDS=72)
      REAL XGAUS(XGRIDS),YGAUS(YGRIDS)
      REAL XRNG(XGRIDS),YRNG(YGRIDS)

C  The grid in radians; note that the latitudes dmu, where mu is the sine(lat)
C  are not evenly spaced

      PARAMETER (DLAM=2.5)

C  The Gaussian grid variables: GLAT are the latitudes in mu=sine(lat);
C  GWT are the associated weights

      DOUBLE PRECISION GLATDP(YGRIDS),GWTDP(YGRIDS)
      REAL GLAT(YGRIDS),GWT(YGRIDS)
      REAL GLAT2(YGRIDS),GWT2(YGRIDS)

C  Field variables

      REAL PBAR(XGRIDS,YGRIDS),PBARTRY(XGRIDS,YGRIDS)
      REAL PBARREC(XGRIDS,YGRIDS),PBARREC2(XGRIDS,YGRIDS)
      REAL RVBAR(XGRIDS,YGRIDS),RVBARREC(XGRIDS,YGRIDS)
      REAL DRVBARREC(XGRIDS,YGRIDS),DRVBARDLREC(XGRIDS,YGRIDS)
      REAL DPBARDLREC(XGRIDS,YGRIDS),DRVBARDMREC(XGRIDS,YGRIDS)
      REAL DPBARDMREC(XGRIDS,YGRIDS),RPRIME(XGRIDS,YGRIDS)
      REAL DYDLAM(XGRIDS,YGRIDS),DYDMU(XGRIDS,YGRIDS)
      REAL DYDLAM2(XGRIDS,YGRIDS),DYDMU2(XGRIDS,YGRIDS)
      REAL YSPC(XGRIDS,YGRIDS),MULT3(XGRIDS,YGRIDS)
      REAL MULT1(XGRIDS,YGRIDS),MULT2(XGRIDS,YGRIDS)
      REAL MULTTOT(XGRIDS,YGRIDS),MULTREC(XGRIDS,YGRIDS)
      REAL PPRIMEREC(XGRIDS,YGRIDS),RVPRIMEREC(XGRIDS,YGRIDS)
      REAL DRVPRIMEREC(XGRIDS,YGRIDS),DRVPRIMEDLREC(XGRIDS,YGRIDS)
      REAL DPPRIMEDLREC(XGRIDS,YGRIDS),RVPRIMEDMREC(XGRIDS,YGRIDS)
      REAL DPPRIMEDMREC(XGRIDS,YGRIDS),FORCEREC(XGRIDS,YGRIDS)
      REAL YSPC2(XGRIDS,YGRIDS),DRVPRIMEDMREC(XGRIDS,YGRIDS)
      REAL DPPRIMEDTREC(XGRIDS,YGRIDS),NPPRIMEREC(XGRIDS,YGRIDS)
      REAL TRYREC(XGRIDS,YGRIDS),TRYREC2(XGRIDS,YGRIDS)
      REAL RPRIMEREC(XGRIDS,YGRIDS),MULT4(XGRIDS,YGRIDS)
      
C  Truncation

      PARAMETER(LIM=25)     ! Number of spherical harmonics (-1)

C  Fourier Field variables

      PARAMETER (FFTP=256)
      INTEGER FFTPP
      REAL PBARFFT(2*LIM,YGRIDS),MULTFFT1S(2*LIM,YGRIDS)
      REAL MULTFFT2S(2*LIM,YGRIDS),RPRIMEFFT(2*LIM,YGRIDS)
      REAL MULTFFT3S(2*LIM,YGRIDS),MULTFFTTOTS(2*LIM,YGRIDS)
      REAL PBARFFTS(2*LIM,YGRIDS),PBARFFTC(2*LIM,YGRIDS)
      REAL RPRIMEFFTS(2*LIM,YGRIDS),RPRIMEFFTC(2*LIM,YGRIDS)
      REAL MULTFFT2C(2*LIM,YGRIDS),MULTFFT1C(2*LIM,YGRIDS)
      REAL MULTFFT3C(2*LIM,YGRIDS),MULTFFTTOTC(2*LIM,YGRIDS)
      REAL PPRIMEFFTS(2*LIM,YGRIDS),PPRIMEFFTC(2*LIM,YGRIDS)
      REAL NPPRIMEFFTS(2*LIM,YGRIDS),NPPRIMEFFTC(2*LIM,YGRIDS)
      REAL TRYFFTS(2*LIM,YGRIDS),TRYFFTC(2*LIM,YGRIDS)
      REAL RVBARFFTS(2*LIM,YGRIDS),RVBARFFTC(2*LIM,YGRIDS)
      
C  Spectral Field variables

      REAL DEPS(LIM+2,LIM+2)
      REAL NRM
      REAL LEG,DLEG,DLEG1
      DIMENSION LEG(0:LIM,0:LIM,1:YGRIDS)
      DIMENSION DLEG(0:LIM,0:LIM,1:YGRIDS)
      DIMENSION DLEG1(0:LIM,0:LIM,1:YGRIDS)
      DOUBLE PRECISION ZS(LIM)
      REAL PBARLEGFFTS(2*LIM,LIM),RVBARLEGFFTS(2*LIM,LIM)
      REAL PBARLEGFFTC(2*LIM,LIM),RVBARLEGFFTC(2*LIM,LIM)
      REAL DRVBARLEGFFTS(2*LIM,LIM),DPBARDLAMS(2*LIM,LIM)
      REAL DRVBARDLAMS(2*LIM,LIM),RPRIMELEGFFTS(2*LIM,LIM)
      REAL PPRIMELEGFFT(2*LIM,LIM),RVPRIMELEGFFT(2*LIM,LIM)
      REAL RVPRIMELEGFFTS(2*LIM,LIM),RVPRIMELEGFFTC(2*LIM,LIM)
      REAL DRVBARLEGFFTC(2*LIM,LIM),DPBARDLAMC(2*LIM,LIM)
      REAL DRVBARDLAMC(2*LIM,LIM),RPRIMELEGFFTC(2*LIM,LIM)
      REAL MULTLEGFFTS(2*LIM,LIM),PPRIMELEGFFT2S(2*LIM,LIM)
      REAL DRVPRIMELEGFFTS(2*LIM,LIM),DPPRIMEDLAMS(2*LIM,LIM)
      REAL DRVPRIMEDLAMS(2*LIM,LIM),AVGRVPRIMELEGFFT2S(2*LIM,LIM)
      REAL DRVPRIMEDTLEGFFTS(2*LIM,LIM),HOLDS(2*LIM,LIM)
      REAL MULTLEGFFTC(2*LIM,LIM),PPRIMELEGFFT2C(2*LIM,LIM)
      REAL DRVPRIMELEGFFTC(2*LIM,LIM),DPPRIMEDLAMC(2*LIM,LIM)
      REAL DRVPRIMEDLAMC(2*LIM,LIM),AVGRVPRIMELEGFFT2C(2*LIM,LIM)
      REAL DRVPRIMEDTLEGFFTC(2*LIM,LIM),HOLDC(2*LIM,LIM)
      REAL PPRIMELEGFFTS(2*LIM,LIM),PPRIMELEGFFTC(2*LIM,LIM)
      REAL MULTLEGFFT1S(2*LIM,LIM),MULTLEGFFT1C(2*LIM,LIM)
      REAL MULTLEGFFT2S(2*LIM,LIM),MULTLEGFFT2C(2*LIM,LIM)
      REAL MULTLEGFFT3S(2*LIM,LIM),MULTLEGFFT3C(2*LIM,LIM)
      REAL TRYLEGFFTS(2*LIM,LIM),TRYLEGFFTC(2*LIM,LIM)
      REAL RVPRIMELEGFFT2S(2*LIM,LIM),RVPRIMELEGFFT2C(2*LIM,LIM)
      REAL DRVPRIMELEGFFT2S(2*LIM,LIM),DRVPRIMELEGFFT2C(2*LIM,LIM)            
      
C  Reverse Legendre variables

      REAL PBARREC1S(2*LIM,YGRIDS),RVBARREC1S(2*LIM,YGRIDS)
      REAL PBARREC1C(2*LIM,YGRIDS),RVBARREC1C(2*LIM,YGRIDS)
      REAL DRVBARREC1S(2*LIM,YGRIDS),DRVBARDLREC1S(2*LIM,YGRIDS)
      REAL DPBARDLREC1S(2*LIM,YGRIDS),DRVBARDMREC1S(2*LIM,YGRIDS)
      REAL DPBARDMREC1S(2*LIM,YGRIDS),PPRIMEREC1(2*LIM,YGRIDS)
      REAL RVPRIMEREC1(2*LIM,YGRIDS),DRVBARREC1C(2*LIM,YGRIDS)
      REAL DPBARDLREC1C(2*LIM,YGRIDS),DRVBARDLREC1C(2*LIM,YGRIDS)
      REAL DPBARDMREC1C(2*LIM,YGRIDS),DRVBARDMREC1C(2*LIM,YGRIDS)
      REAL RVPRIMEREC1S(2*LIM,YGRIDS),PPRIMEREC1S(2*LIM,YGRIDS)
      REAL RVPRIMEREC1C(2*LIM,YGRIDS),PPRIMEREC1C(2*LIM,YGRIDS)
      REAL DRVPRIMEREC1S(2*LIM,YGRIDS),DRVPRIMEREC1C(2*LIM,YGRIDS)
      REAL DRVPRIMEDLREC1S(2*LIM,YGRIDS),DRVPRIMEDLREC1C(2*LIM,YGRIDS)
      REAL DPPRIMEDLREC1S(2*LIM,YGRIDS),DPPRIMEDLREC1C(2*LIM,YGRIDS)
      REAL DRVPRIMEDMREC1S(2*LIM,YGRIDS),DRVPRIMEDMREC1C(2*LIM,YGRIDS)
      REAL DPPRIMEDMREC1S(2*LIM,YGRIDS),DPPRIMEDMREC1C(2*LIM,YGRIDS)
      REAL FORCEREC1S(YGRIDS),FORCEREC1C(YGRIDS)
      REAL DPPRIMEDTREC1S(2*LIM,YGRIDS),DPPRIMEDTREC1C(2*LIM,YGRIDS)
      REAL MULTREC1S(2*LIM,YGRIDS),MULTREC1C(2*LIM,YGRIDS)
      REAL TRYREC1S(2*LIM,YGRIDS),TRYREC1C(2*LIM,YGRIDS)
      REAL RPRIMEREC1S(2*LIM,YGRIDS),RPRIMEREC1C(2*LIM,YGRIDS)      
      
C  Domain variables

      PARAMETER (RADI=6.37E6)
      REAL F0,BETA,F(YGRIDS),OMEGA,PI
      REAL X0,Y0,D0,DELX,DELY,ALPH,OM,ALP,EPSA
      REAL R(XGRIDS,YGRIDS)

C  Spare Declarations

      REAL FFTINPUT(FFTP),FFTOUTPUT(FFTP),RFFTOUTPUT(FFTP)
      INTEGER I,J,K,II,JJ,KK,PLIM,A,B,C,M,N,FT,M1,N1,INFO,WCNT,TOTALT,DT
      INTEGER FILT,DCNT
      REAL AA,BB,CC,CX,BBB,CCC,CA,CB,CD,AZ,BZ,CZ,DZ,CE,CF,CG
      REAL AR,BR,CR,DR
      DOUBLE PRECISION DPA,DPB,DPC
      real ug(xgrids,lim)

C  File Path

      CHARACTER*38 FPATH
      CHARACTER*3 OUTCH

C  Output Path

      FPATH='/home/jshaman/Hurricanes/BTmodel/Temp/'

C  Getting pi,i 

      PI=2.*ACOS(0.)
      CX=(0,1.0)  
c      RADI=6.37E6
      
C  Loading in the streamfunction (and vorticity) files interpolated onto
C  the Gaussian latitude grid.   NOTE: the orientation is VERY important.
C  The order of the DO loops here (latitude, nested in longitude) is made 
C  to match the Matlab driver file ('steady_lin_BT_f77driver.m') output 
C  of the interpolated streamfunction and relative vorticity fields.
C  PBAR(1:XGRIDS,:,:) is west to east; PBAR(:,1:YGRIDS,:) is north to south.

      OPEN(21,FILE=FPATH//'startPbar')
      OPEN(22,FILE=FPATH//'startRVbar')
      DO I=1,XGRIDS
         DO J=1,YGRIDS
            READ(21,*) PBAR(I,J)
            READ(22,*) RVBAR(I,J)
         END DO
      END DO
      CLOSE(22)
      CLOSE(21)

C  Loading the Gaussian latitudes (sin(phi)=mu), which go from north to south
C  and their associated Gaussian weights

      OPEN(23,FILE=FPATH//'glatswts')
      DO J=1,YGRIDS
         READ(23,*) GLATDP(J),GWTDP(J)
         GLAT(J)=REAL(GLATDP(J))
         GWT(J)=REAL(GWTDP(J))
      END DO
      CLOSE(23)

10         FORMAT(E17.8)

C  Fourier transform by hand (agrees with by hand and FFT in Matlab
C  The FFT.f from numerical recipes differs
      
      DO J=1,YGRIDS
         DO I=1,lim
            PBARFFTS(I,J)=0.
            PBARFFTC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do ii=0,xgrids-1
            do jj=1,ygrids
               if(m.eq.0)then
                  bb=Pbar(ii+1,jj)*cos(m*pi*(ii+1)*2/xgrids)/xgrids
                  pbarfftc(m+1,jj)=pbarfftc(m+1,jj)+bb
               else
                  bb=Pbar(ii+1,jj)*cos(m*pi*(ii+1)*2/xgrids)*2/xgrids
                  pbarfftc(m+1,jj)=pbarfftc(m+1,jj)+bb
                  aa=Pbar(ii+1,jj)*sin(m*pi*(ii+1)*2/xgrids)*2/xgrids
                  pbarffts(m+1,jj)=pbarffts(m+1,jj)+aa
               end if
             end do
         end do
      end do

C  Next putting in the epsilon functions sqrt((n^2-m^2)/(4*n^2-1))

      DO J=0,LIM+1
         DO I=0,J
        DEPS(I+1,J+1)=((REAL(J)**2-REAL(I)**2)/(4*REAL(J)**2-1))**0.5
         END DO
      END DO

C  Next getting the associated legendre functions and their derivatives d/dmu
C  When MLPMN is called it returns for a given Gaussian latitude [-1 1]
C  all the n associated legendre polynomials, for n=0 to N and m=0 to N
C  this includes those for which m>n, which are zero.
C  Then only have to loop through the latitudes

C  Both the Gaussian weights and the normalized legendre polynomials
C  agree with Matlab
 
C  Getting values for 0:LIM, so there are LIM+1 values

      PLIM=LIM
      DO KK=1,YGRIDS
         CALL MLPMN2(LIM,LIM,GLATDP(KK),PM,PD)
         DO II=0,LIM
            DO JJ=0,LIM
               LEG(II,JJ,KK)=PM(II,JJ)
               DLEG(II,JJ,KK)=PD(II,JJ)
            END DO
         END DO
      END DO   

C  Getting the spherical coefficients by Gaussian quadrature

      OPEN(24,FILE=FPATH//'PbarlegfftS')
      OPEN(84,FILE=FPATH//'PbarlegfftC')
      DO II=0,LIM-1
         DO JJ=ABS(II),LIM-1
            DO KK=1,YGRIDS
               IF(KK.EQ.1) THEN
                  AA=0.
                  BB=0.
               END IF
               if(ii.lt.0)then
                 ca=1.
                 cb=1.
                 cc=jj-abs(ii)
                 if(cc.gt.1)then
                 do i=1,cc
                   ca=ca*i
                 end do
                 end if
                 cc=jj+abs(ii)
                 if(cc.gt.1)then
                 do i=1,cc
                   cb=cb*i
                 end do
                 end if
                 br=leg(abs(ii),jj,kk)*gwt(kk)*((-1)**ii)*ca/cb
               else
                 BR=LEG(ABS(II),JJ,KK)*GWT(KK)
               end if                        
               AA=AA+BR*PBARFFTS(II+1,KK)   
               BB=BB+BR*PBARFFTC(II+1,KK)
            END DO
            PBARLEGFFTS(II+1,JJ+1)=AA
            PBARLEGFFTC(II+1,JJ+1)=BB
            WRITE(24,10)AA
            WRITE(84,10)BB
         END DO
      END DO
      CLOSE(24)
      CLOSE(84)

C  The above multiplications appear correct

C  Performing the reverse Legendre transform on the basic field

      do j=1,ygrids
         do i=0,lim-1
            pbarrec1C(i+1,j)=0
            pbarrec1S(i+1,j)=0
         end do
      end do

      DO KK=1,YGRIDS
         DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
               AA=PBARLEGFFTS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               PBARREC1S(II+1,KK)=PBARREC1S(II+1,KK)+AA
               BB=PBARLEGFFTC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               PBARREC1C(II+1,KK)=PBARREC1C(II+1,KK)+BB
            END DO
         END DO
      END DO

      OPEN(25,FILE=FPATH//'P1S')
      OPEN(85,FILE=FPATH//'P1C')
      OPEN(26,FILE=FPATH//'P2')
      do m=0,lim-1
         do jj=1,ygrids
            write(25,10)real(pbarrec1S(m+1,jj))
            write(85,10)real(pbarrec1C(m+1,jj))
            write(26,10)real(pbarfft(m+1,jj))
         end do
      end do
      CLOSE(25)
      CLOSE(85)
      CLOSE(26)

C  The reconstruction matches the original Fourier transform

C  Performing the reverse Fourier transform

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            PBARREC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do j=1,ygrids
            do i=1,xgrids
               AA=PBARREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=PBARREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               PBARREC(I,J)=PBARREC(I,J)+AA+BB
            END DO
         END DO
      END DO

      OPEN(27,FILE=FPATH//'Pbarrec')
      DO J=1,YGRIDS
         DO I=1,XGRIDS
            WRITE(27,10)PBARREC(I,J)
         END DO
      END DO
      CLOSE(27)

C  There are 6 fields needed:

C  1) Del2(Psi_mn), = -n(n+1)/a^2*Psi_mn = RelV_mn
C  2) d/dmu of 1,   = RelV_mn*[-n*Eps_mn+1*Y_mn+1 + (n+1)*Eps_mn*Y_mn-1]
C       This will be off by a factor of (1-mu^2) when put into grid space
C  3) d/dlambda of 1,  = im*RelV_mn
C  4) d/dmu of Pbar,   = Pbar_mn*[-n*Eps_mn+1*Y_mn+1 + (n+1)*Eps_mn*Y_mn-1]
C       This will be off by a factor of (1-mu^2) when put into grid space
C  5) d/dlambda of Pbar,  = im*Pbar_mn
C  6) Del4(RelV_mn) = (-n(n+1)/a^2)^2*RelV_mn 

C  Starting with 1 and 6 in spectral space

      DO M=0,LIM-1
         DO N=ABS(M),LIM-1
      RVBARLEGFFTS(m+1,n+1)=-n*(n+1)*PbarlegfftS(m+1,n+1)/RADI/RADI
      DRVBARLEGFFTS(m+1,n+1)=(-n*(n+1))**2*
     1     RVbarlegfftS(m+1,n+1)/(RADI)**4
      RVBARLEGFFTC(m+1,n+1)=-n*(n+1)*PbarlegfftC(m+1,n+1)/RADI/RADI
      DRVBARLEGFFTC(m+1,n+1)=(-n*(n+1))**2*
     1     RVbarlegfftC(m+1,n+1)/(RADI)**4
            END DO
      END DO


CCCCCCCCCCCCCCCCCCCCCCC
CC  Testing controlled RV fields
      
      aa=0
      if(aa.eq.1)then
      DO J=1,YGRIDS
         DO I=1,lim
            RVBARFFTS(I,J)=0.
            RVBARFFTC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do ii=0,xgrids-1
            do jj=1,ygrids
               if(m.eq.0)then
                  bb=Rvbar(ii+1,jj)*cos(m*pi*(ii+1)*2/xgrids)/xgrids
                  rvbarfftc(m+1,jj)=rvbarfftc(m+1,jj)+bb                  
               else
                  bb=Rvbar(ii+1,jj)*cos(m*pi*(ii+1)*2/xgrids)*2/xgrids
                  rvbarfftc(m+1,jj)=rvbarfftc(m+1,jj)+bb
                  aa=Rvbar(ii+1,jj)*sin(m*pi*(ii+1)*2/xgrids)*2/xgrids
                  rvbarffts(m+1,jj)=rvbarffts(m+1,jj)+aa
               end if
            end do
         end do
      end do

      DO II=0,LIM-1
         DO JJ=ABS(II),LIM-1
            DO KK=1,YGRIDS
               IF(KK.EQ.1) THEN
                  AA=0.
                  BB=0.
               END IF
               if(ii.lt.0)then
                 ca=1.
                 cb=1.
                 cc=jj-abs(ii)
                 if(cc.gt.1)then
                 do i=1,cc
                   ca=ca*i
                 end do
                 end if
                 cc=jj+abs(ii)
                 if(cc.gt.1)then
                 do i=1,cc
                   cb=cb*i
                 end do
                 end if
                 br=leg(abs(ii),jj,kk)*gwt(kk)*((-1)**ii)*ca/cb
               else
                 BR=LEG(ABS(II),JJ,KK)*GWT(KK)
               end if                        
               AA=AA+BR*RVBARFFTS(II+1,KK)   
               BB=BB+BR*RVBARFFTC(II+1,KK)
            END DO
            RVBARLEGFFTS(II+1,JJ+1)=AA
            RVBARLEGFFTC(II+1,JJ+1)=BB
         END DO
      END DO

      do j=1,ygrids
         do i=0,lim-1
            rvbarrec1C(i+1,j)=0
            rvbarrec1S(i+1,j)=0
         end do
      end do

      DO KK=1,YGRIDS
         DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
               AA=RVBARLEGFFTS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               RVBARREC1S(II+1,KK)=RVBARREC1S(II+1,KK)+AA
               BB=RVBARLEGFFTC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               RVBARREC1C(II+1,KK)=RVBARREC1C(II+1,KK)+BB
            END DO
         END DO
      END DO

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            RVBARREC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do j=1,ygrids
            do i=1,xgrids
               AA=RVBARREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=RVBARREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               RVBARREC(I,J)=RVBARREC(I,J)+AA+BB
            END DO
         END DO
      END DO

      OPEN(66,FILE=FPATH//'RVbarrec3')
      DO J=1,YGRIDS
         DO I=1,XGRIDS
            WRITE(66,10)RVBARREC(I,J)
         END DO
      END DO
      CLOSE(66)

      DO M=0,LIM-1
         DO N=ABS(M),LIM-1
            DRVBARLEGFFTS(m+1,n+1)=(-n*(n+1))**2*
     1     RVbarlegfftS(m+1,n+1)/(RADI)**4
            DRVBARLEGFFTC(m+1,n+1)=(-n*(n+1))**2*
     1     RVbarlegfftC(m+1,n+1)/(RADI)**4
            IF(N.GT.0)THEN
            AR=-1*RADI*RADI/N/(N+1)
            PBARLEGFFTS(M+1,N+1)=RVBARLEGFFTS(M+1,N+1)*AR
            PBARLEGFFTC(M+1,N+1)=RVBARLEGFFTC(M+1,N+1)*AR
            else
              ar=-1*radi*radi
            PBARLEGFFTS(M+1,N+1)=0 !RVBARLEGFFTS(M+1,N+1)*AR
            PBARLEGFFTC(M+1,N+1)=0 !RVBARLEGFFTC(M+1,N+1)*AR
            end if
            
            END DO
      END DO

      end if
        
C  Performing the reverse Legendre transform on these relative vorticity 
C  coefficients

      do j=1,ygrids
         do i=0,lim-1
            rvbarrec1S(i+1,j)=0
            drvbarrec1S(i+1,j)=0
            rvbarrec1C(i+1,j)=0
            drvbarrec1C(i+1,j)=0
         end do
      end do

      DO KK=1,YGRIDS
         DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
               AA=RVBARLEGFFTS(II+1,JJ+1)*LEG(ABS(ii),jj,KK)
               RVBARREC1S(II+1,KK)=RVBARREC1S(II+1,KK)+AA
               AA=DRVBARLEGFFTS(II+1,JJ+1)*LEG(ABS(ii),jj,KK)
               DRVBARREC1S(II+1,KK)=DRVBARREC1S(II+1,KK)+AA
               AA=RVBARLEGFFTC(II+1,JJ+1)*LEG(ABS(ii),jj,KK)
               RVBARREC1C(II+1,KK)=RVBARREC1C(II+1,KK)+AA
               AA=DRVBARLEGFFTC(II+1,JJ+1)*LEG(ABS(ii),jj,KK)
               DRVBARREC1C(II+1,KK)=DRVBARREC1C(II+1,KK)+AA
            END DO
         END DO
      END DO

C  Performing the reverse Fourier transform to get these relative vorticity
C  variables in grid space

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            RVBARREC(I,J)=0.
            DRVBARREC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do j=1,ygrids
            do i=1,xgrids
               AA=RVBARREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=RVBARREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               RVBARREC(I,J)=RVBARREC(I,J)+AA+BB
               AA=DRVBARREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DRVBARREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DRVBARREC(I,J)=DRVBARREC(I,J)+AA+BB
            END DO
         END DO
      END DO

      OPEN(28,FILE=FPATH//'RVbarrec')
      OPEN(38,FILE=FPATH//'DRVbarrec')
      DO J=1,YGRIDS
         DO I=1,XGRIDS
            WRITE(28,10)real(RVBARREC(I,J))
            WRITE(38,10)real(DRVBARREC(I,J))
         END DO
      END DO
      CLOSE(28)
      CLOSE(38)

C  Now doing 3 and 5 (d/dlambda terms) in spectral space 
C  The below is correct, changing the coefficients to be transformed
C  involves swapping sines and cosines for the correct derivatives

      DO M=0,LIM-1
         DO N=ABS(M),LIM-1
            DPBARDLAMS(m+1,n+1)=-1*M*PbarlegfftC(m+1,n+1)
            DPBARDLAMC(m+1,n+1)=M*PbarlegfftS(m+1,n+1)
            DRVBARDLAMC(m+1,n+1)=M*RVbarlegfftS(m+1,n+1)
            DRVBARDLAMS(m+1,n+1)=-1*M*RVbarlegfftC(m+1,n+1)
         END DO
      END DO

C  Performing the reverse Legendre transform on these relative vorticity 
C  coefficients

      do j=1,ygrids
         do i=1,lim
            dpbardlrec1S(i,j)=0.
            drvbardlrec1S(i,j)=0.
            dpbardlrec1C(i,j)=0.
            drvbardlrec1C(i,j)=0.
         end do
      end do

      DO KK=1,YGRIDS
         DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
               AA=DPBARDLAMS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               DPBARDLREC1S(II+1,KK)=DPBARDLREC1S(II+1,KK)+AA
               AA=DRVBARDLAMS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               DRVBARDLREC1S(II+1,KK)=DRVBARDLREC1S(II+1,KK)+AA
               AA=DPBARDLAMC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               DPBARDLREC1C(II+1,KK)=DPBARDLREC1C(II+1,KK)+AA
               AA=DRVBARDLAMC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               DRVBARDLREC1C(II+1,KK)=DRVBARDLREC1C(II+1,KK)+AA
            END DO
         END DO
      END DO

C  Performing the reverse Fourier transform to get these relative vorticity
C  variables in grid space

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            DPBARDLREC(I,J)=0.
            DRVBARDLREC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do j=1,ygrids
            do i=1,xgrids
               AA=DPBARDLREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DPBARDLREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DPBARDLREC(I,J)=DPBARDLREC(I,J)+AA+BB
               AA=DRVBARDLREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DRVBARDLREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DRVBARDLREC(I,J)=DRVBARDLREC(I,J)+AA+BB
            END DO
         END DO
      END DO

C  Now doing 2 and 4 (d/dmu terms) in spectral space
C  Getting the (1-mu^2)*dPmn/dmu 

      DO KK=1,YGRIDS
         do m=0,lim
           do n=m,lim
               IF(N.EQ.0)THEN
                  DLEG1(M,N,KK)=-1*N*DEPS(M+1,N+2)*LEG(M,N+1,KK)
               ELSE
                  DLEG1(M,N,KK)=-1*N*DEPS(M+1,N+2)*LEG(M,N+1,KK)+
     1                 (N+1)*DEPS(M+1,N+1)*LEG(M,N-1,KK)
               END IF
            END DO
         END DO
      END DO

C  Performing the reverse Legendre transform on these relative vorticity 
C  coefficients

      do j=1,ygrids
         do i=1,lim
            dpbardmrec1S(i,j)=0.
            drvbardmrec1S(i,j)=0.
            dpbardmrec1C(i,j)=0.
            drvbardmrec1C(i,j)=0.
         end do
      end do

      DO KK=1,YGRIDS
         AR=1-GLAT(KK)**2          !  1-MU^2 Factor
         DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
               AA=PBARLEGFFTS(II+1,JJ+1)*DLEG1(ABS(II),JJ,KK)/AR
               DPBARDMREC1S(II+1,KK)=DPBARDMREC1S(II+1,KK)+AA
               AA=RVBARLEGFFTS(II+1,JJ+1)*DLEG1(ABS(II),JJ,KK)/AR
               DRVBARDMREC1S(II+1,KK)=DRVBARDMREC1S(II+1,KK)+AA
               AA=PBARLEGFFTC(II+1,JJ+1)*DLEG1(ABS(II),JJ,KK)/AR
               DPBARDMREC1C(II+1,KK)=DPBARDMREC1C(II+1,KK)+AA
               AA=RVBARLEGFFTC(II+1,JJ+1)*DLEG1(ABS(II),JJ,KK)/AR
               DRVBARDMREC1C(II+1,KK)=DRVBARDMREC1C(II+1,KK)+AA
            END DO
         END DO
      END DO

C  Performing the reverse Fourier transform to get these relative vorticity
C  variables in grid space

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            DPBARDMREC(I,J)=0.
            DRVBARDMREC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do j=1,ygrids
            do i=1,xgrids
               AA=DPBARDMREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DPBARDMREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DPBARDMREC(I,J)=DPBARDMREC(I,J)+AA+BB
               AA=DRVBARDMREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DRVBARDMREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DRVBARDMREC(I,J)=DRVBARDMREC(I,J)+AA+BB
            END DO
         END DO
      END DO

      OPEN(40,FILE=FPATH//'dpdl')
      OPEN(41,FILE=FPATH//'dpdm')
      OPEN(42,FILE=FPATH//'drdl')
      OPEN(43,FILE=FPATH//'drdm')
      DO J=1,YGRIDS
         DO I=1,XGRIDS
            WRITE(40,10)dpbardlrec(i,j)
            WRITE(41,10)dpbardmrec(i,j)
            WRITE(42,10)drvbardlrec(i,j)
            WRITE(43,10)drvbardmrec(i,j)
         END DO
      END DO
      CLOSE(40)
      CLOSE(41)
      CLOSE(42)
      CLOSE(43)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC  Now getting the forcing

C  The forcing, R, 
C  is f*D0*(1-[(lambda-lambda0)^2/dlamb^2 + (phi-phi0)^2/dphi^2]^0.5)
C  where f is 2*Omega*sin(phi)
C  D0 is the divergence, set to 3e-6/s, it is ~8mm of rain/day (HK81)
C  lambda is longitude and lambda0 is the center of the forcing
C  phi is longitude and phi0 is the center of the forcing
C  dlamb and dphi are shape factors--30 and 25 degrees in standard CCM
C    simulations corresponding to the areal extent of tropical vorticity
C    anomalies


      F0=2*7.292E-5
      DO I=1,YGRIDS
         F(I)=F0*GLAT(I)
         YRNG(I)=ASIN(GLAT(I))
      END DO
      DO J=1,XGRIDS
         XRNG(J)=J*2*PI/XGRIDS
      END DO
     
C  The forcing location

      X0=120*PI/180    
      Y0=0*PI/180      
      DELY=20*pi/180
      DELX=20*pi/180
      D0=3e-6;

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            AR=(XRNG(I)-X0)**2/DELX**2+(YRNG(J)-Y0)**2/DELY**2
            BR=F(J)+RVBARREC(I,J)
            IF(AR.LE.1)THEN
               R(I,J)=-BR*D0  !*(1-AR**(0.5))
            END IF
         END DO
      END DO

      OPEN(29,FILE=FPATH//'Rawforcing')
      DO J=1,YGRIDS
         DO I=1,XGRIDS
            WRITE(29,10)R(I,J)
         END DO
      END DO
      CLOSE(29)

C  The forcing plus the background mean flow

      ALPH=1.57e-6      !  This is in units of 1/s, equals ~1/7days
      OM=7.292e-5
      ALP=2.34e16      !  From Grose and Hoskins, units m^4/s
c      ALP=0.

      DO J=1,YGRIDS
         DR=1-GLAT(J)**2  
         DO I=1,XGRIDS
            AR=DPBARDMREC(I,J)*DRVBARDLREC(I,J)/DR
            BR=DPBARDLREC(I,J)*(DRVBARDMREC(I,J))/DR
            CR=(AR-BR)/RADI**2-DPBARDLREC(I,J)*2*OM/RADI**2  
            RPRIME(I,J)=R(I,J)
         END DO
      END DO

      OPEN(30,FILE=FPATH//'Fullforcing')
      DO J=1,YGRIDS
         DO I=1,XGRIDS
            WRITE(30,10)RPRIME(I,J)
         END DO
      END DO
      CLOSE(30)

C  Fourier transform of the forcing

      DO J=1,YGRIDS
         DO I=1,lim
            RPRIMEFFTS(I,J)=0.
            RPRIMEFFTC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do ii=0,xgrids-1
            do jj=1,ygrids
               if(m.eq.0)then
                  bb=Rprime(ii+1,jj)*cos(m*pi*(ii+1)*2/xgrids)/xgrids
                  rprimefftc(m+1,jj)=rprimefftc(m+1,jj)+bb
               else
                  bb=Rprime(ii+1,jj)*cos(m*pi*(ii+1)*2/xgrids)*2/xgrids
                  rprimefftc(m+1,jj)=rprimefftc(m+1,jj)+bb
                  aa=Rprime(ii+1,jj)*sin(m*pi*(ii+1)*2/xgrids)*2/xgrids
                  rprimeffts(m+1,jj)=rprimeffts(m+1,jj)+aa                  
               end if
            end do
         end do
      end do

C  Getting the spherical coefficients of the forcing by Gaussian quadrature

      DO II=0,LIM-1
         DO JJ=ABS(II),LIM-1
            DO KK=1,YGRIDS
               IF(KK.EQ.1) THEN
                  AA=0.
                  BB=0.
               END IF
               if(ii.lt.0)then
                 ca=1.
                 cb=1.
                 cc=jj-abs(ii)
                 if(cc.gt.1)then
                 do i=1,cc
                   ca=ca*i
                 end do
                 end if
                 cc=jj+abs(ii)
                 if(cc.gt.1)then
                 do i=1,cc
                   cb=cb*i
                 end do
                 end if
                 br=leg(abs(ii),jj,kk)*gwt(kk)*((-1)**ii)*ca/cb
               else
                 BR=LEG(ABS(II),JJ,KK)*GWT(KK)
               end if                        
               AA=AA+BR*RPRIMEFFTS(II+1,KK)   
               BB=BB+BR*RPRIMEFFTC(II+1,KK)
            END DO
            RPRIMELEGFFTS(II+1,JJ+1)=AA
            RPRIMELEGFFTC(II+1,JJ+1)=BB
         END DO
      END DO
      
      
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC  Now have the mean flow and forcing; can solve for the anomaly vorticity

      WCNT=100
      DCNT=100
      DT=3600         !  6 minute time step
      TOTALT=86400*50
      OPEN(54,FILE=FPATH//'UPDATE1')
      OPEN(53,FILE=FPATH//'UPDATE2')
      OPEN(77,FILE=FPATH//'mult')
      OPEN(78,FILE=FPATH//'dzetadx')
      OPEN(79,FILE=FPATH//'gridupdate')
      
CC  Trying to base on relative vorticity        
        
      DO T=0,TOTALT,DT
        IF(T.EQ.0)THEN
          DO M=0,LIM-1
            DO N=ABS(M),LIM-1
              RVPRIMELEGFFTS(M+1,N+1)=2*DT*RPRIMELEGFFTS(M+1,N+1)
              RVPRIMELEGFFT2S(M+1,N+1)=2*DT*RPRIMELEGFFTS(M+1,N+1)
              RVPRIMELEGFFTC(M+1,N+1)=2*DT*RPRIMELEGFFTC(M+1,N+1)
              RVPRIMELEGFFT2C(M+1,N+1)=2*DT*RPRIMELEGFFTC(M+1,N+1)
            END DO
          END DO
c        END IF        

C  Here comes the main script
        
        ELSEIF(T.GT.0)THEN

C  There are 6 rfields of Pprime needed (these equal 0 for t=1):

C  1) Del2(Psi_mn), = -n(n+1)/a^2*Psi_mn = RelV_mn
C  2) d/dmu of 1,   = RelV_mn*[-n*Eps_mn+1*Y_mn+1 + (n+1)*Eps_mn*Y_mn-1]
C         This will be off by a factor of (1-mu^2) when put into grid space
C  3) d/dlambda of 1,  = im*RelV_mn
C  4) d/dmu of Pbar,   = Pbar_mn*[-n*Eps_mn+1*Y_mn+1 + (n+1)*Eps_mn*Y_mn-1]
C         This will be off by a factor of (1-mu^2) when put into grid space
C  5) d/dlambda of Pbar,  = im*Pbar_mn
C  6) Del4(RelV_mn) = (-n(n+1)/a^2)^2*RelV_mn

C  Starting with 1 and 6 in spectral space, which again are only needed
C  in spectral space--NOTE: NEED TO APPLY THE BIHARMONIC DIFFUSION TO THE
C  PRIOR TIME STEP       

        DO M=0,LIM-1
          DO N=ABS(M),LIM-1
            DRVPRIMELEGFFTS(m+1,n+1)=((-n*(n+1))**2)*
     1           RVPRIMELEGFFTS(m+1,n+1)/(RADI)**4
            DRVPRIMELEGFFTC(m+1,n+1)=((-n*(n+1))**2)*
     1           RVPRIMELEGFFTC(m+1,n+1)/(RADI)**4
            DRVPRIMELEGFFT2S(m+1,n+1)=((-n*(n+1))**2)*
     1           RVPRIMELEGFFT2S(m+1,n+1)/(RADI)**4
            DRVPRIMELEGFFT2C(m+1,n+1)=((-n*(n+1))**2)*
     1           RVPRIMELEGFFT2C(m+1,n+1)/(RADI)**4
            IF(N.GT.0)THEN
            AR=-1*RADI*RADI/N/(N+1)
            PPRIMELEGFFTS(M+1,N+1)=RVPRIMELEGFFTS(M+1,N+1)*AR
            PPRIMELEGFFT2S(M+1,N+1)=RVPRIMELEGFFT2S(M+1,N+1)*AR
            PPRIMELEGFFTC(M+1,N+1)=RVPRIMELEGFFTC(M+1,N+1)*AR
            PPRIMELEGFFT2C(M+1,N+1)=RVPRIMELEGFFT2C(M+1,N+1)*AR
            else
              ar=-1*radi*radi
            PPRIMELEGFFTS(M+1,N+1)=0 !RVPRIMELEGFFTS(M+1,N+1)*AR
            PPRIMELEGFFT2S(M+1,N+1)=0 !RVPRIMELEGFFT2S(M+1,N+1)*AR
            PPRIMELEGFFTC(M+1,N+1)=0 !RVPRIMELEGFFTC(M+1,N+1)*AR
            PPRIMELEGFFT2C(M+1,N+1)=0 !RVPRIMELEGFFT2C(M+1,N+1)*AR
            end if
          END DO
        END DO

C  Performing the reverse Legendre transform on the current prime fields

        do j=1,ygrids
          do i=0,lim-1
            PPRIMEREC1S(I+1,J)=0.
            PPRIMEREC1C(I+1,J)=0.
            RVPRIMEREC1S(I+1,J)=0.
            RVPRIMEREC1C(I+1,J)=0.
          end do
        end do

        DO KK=1,YGRIDS
          AR=1-GLAT(KK)**2                        !  1-MU^2 Factor
          DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
              AA=PPRIMELEGFFTS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              PPRIMEREC1S(II+1,KK)=PPRIMEREC1S(II+1,KK)+AA
              AA=PPRIMELEGFFTC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              PPRIMEREC1C(II+1,KK)=PPRIMEREC1C(II+1,KK)+AA
              AA=RVPRIMELEGFFTS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              RVPRIMEREC1S(II+1,KK)=RVPRIMEREC1S(II+1,KK)+AA
              AA=RVPRIMELEGFFTC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              RVPRIMEREC1C(II+1,KK)=RVPRIMEREC1C(II+1,KK)+AA
            END DO
          END DO
        END DO
        
C  Performing the reverse Fourier transform

        DO J=1,YGRIDS
          DO I=1,XGRIDS
            PPRIMEREC(I,J)=0.
            RVPRIMEREC(I,J)=0.
          END DO
        END DO

c      do m=1,lim-1  
      do m=0,lim-1
c        do m=-1*lim+1,lim-1
        do j=1,ygrids
            do i=1,xgrids
               AA=PPRIMEREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=PPRIMEREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               PPRIMEREC(I,J)=PPRIMEREC(I,J)+AA+BB
               AA=RVPRIMEREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=RVPRIMEREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               RVPRIMEREC(I,J)=RVPRIMEREC(I,J)+AA+BB
            END DO
         END DO
      END DO

C  Now doing 3 and 5 (d/dlambda terms) in spectral space 
C  The below is correct, changing the coefficients to be transformed
C  involves swapping sines and cosines for the correct derivatives

      DO M=0,LIM-1
         DO N=ABS(M),LIM-1
            DPPRIMEDLAMS(m+1,n+1)=-1*M*PprimelegfftC(m+1,n+1)
            DPPRIMEDLAMC(m+1,n+1)=M*PprimelegfftS(m+1,n+1)
            DRVPRIMEDLAMC(m+1,n+1)=M*RVprimelegfftS(m+1,n+1)
            DRVPRIMEDLAMS(m+1,n+1)=-1*M*RVprimelegfftC(m+1,n+1)
         END DO
      END DO      

C  Performing the reverse Legendre transform on these relative vorticity 
C  coefficients

      do j=1,ygrids
         do i=0,lim-1
            dpprimedlrec1S(i+1,j)=0.
            drvprimedlrec1S(i+1,j)=0.
            dpprimedlrec1C(i+1,j)=0.
            drvprimedlrec1C(i+1,j)=0.
         end do
      end do

      DO KK=1,YGRIDS
         DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
               AA=DPPRIMEDLAMS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               DPPRIMEDLREC1S(II+1,KK)=DPPRIMEDLREC1S(II+1,KK)+AA
               AA=DRVPRIMEDLAMS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               DRVPRIMEDLREC1S(II+1,KK)=DRVPRIMEDLREC1S(II+1,KK)+AA
               AA=DPPRIMEDLAMC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               DPPRIMEDLREC1C(II+1,KK)=DPPRIMEDLREC1C(II+1,KK)+AA
               AA=DRVPRIMEDLAMC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
               DRVPRIMEDLREC1C(II+1,KK)=DRVPRIMEDLREC1C(II+1,KK)+AA
            END DO
         END DO
      END DO

C  Performing the reverse Fourier transform to get these relative vorticity
C  variables in grid space

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            DPPRIMEDLREC(I,J)=0.
            DRVPRIMEDLREC(I,J)=0.
         END DO
      END DO

c        do m=-1*lim+1,lim-1
c      do m=1,lim-1
      do m=0,lim-1
         do j=1,ygrids
            do i=1,xgrids
               AA=DPPRIMEDLREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DPPRIMEDLREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DPPRIMEDLREC(I,J)=DPPRIMEDLREC(I,J)+AA+BB
               AA=DRVPRIMEDLREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DRVPRIMEDLREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DRVPRIMEDLREC(I,J)=DRVPRIMEDLREC(I,J)+AA+BB
            END DO
         END DO
      END DO

C  Performing the reverse Legendre transform on these relative vorticity 
C  coefficients

      do j=1,ygrids
         do i=0,lim-1
            dpprimedmrec1S(i+1,j)=0.
            drvprimedmrec1S(i+1,j)=0.
            dpprimedmrec1C(i+1,j)=0.
            drvprimedmrec1C(i+1,j)=0.
         end do
      end do

      DO KK=1,YGRIDS
         AR=1-GLAT(KK)**2          !  1-MU^2 Factor
         DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
               AA=PPRIMELEGFFTS(II+1,JJ+1)*DLEG1(ABS(II),JJ,KK)/AR
               DPPRIMEDMREC1S(II+1,KK)=DPPRIMEDMREC1S(II+1,KK)+AA
               AA=RVPRIMELEGFFTS(II+1,JJ+1)*DLEG1(ABS(II),JJ,KK)/AR
               DRVPRIMEDMREC1S(II+1,KK)=DRVPRIMEDMREC1S(II+1,KK)+AA
               AA=PPRIMELEGFFTC(II+1,JJ+1)*DLEG1(ABS(II),JJ,KK)/AR
               DPPRIMEDMREC1C(II+1,KK)=DPPRIMEDMREC1C(II+1,KK)+AA
               AA=RVPRIMELEGFFTC(II+1,JJ+1)*DLEG1(ABS(II),JJ,KK)/AR
               DRVPRIMEDMREC1C(II+1,KK)=DRVPRIMEDMREC1C(II+1,KK)+AA
            END DO
         END DO
      END DO

C  Performing the reverse Fourier transform to get these relative vorticity
C  variables in grid space

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            DPPRIMEDMREC(I,J)=0.
            DRVPRIMEDMREC(I,J)=0.
         END DO
      END DO

      do m=0,lim-1
         do j=1,ygrids
            do i=1,xgrids
               AA=DPPRIMEDMREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DPPRIMEDMREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DPPRIMEDMREC(I,J)=DPPRIMEDMREC(I,J)+AA+BB
               AA=DRVPRIMEDMREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=DRVPRIMEDMREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               DRVPRIMEDMREC(I,J)=DRVPRIMEDMREC(I,J)+AA+BB
            END DO
         END DO
      END DO

CC
CC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC  Everything needed for multiplication is now in grid space
CC
CC      

C  Multiplying the terms J(Pbar,Rvprime) and J(Pprime, Rvbar)

            DO J=1,YGRIDS
               DO I=1,XGRIDS
                 aa=1e-4
                MULT1(I,J)=(DPBARDMREC(I,J)*DRVPRIMEDLREC(I,J)-
     1                 DPBARDLREC(I,J)*DRVPRIMEDMREC(I,J))/RADI/RADI
              MULT2(I,J)=(DPPRIMEDMREC(I,J)*DRVBARDLREC(I,J)-
     1                 DPPRIMEDLREC(I,J)*(DRVBARDMREC(I,J)+2*OM))/
     2                 RADI/RADI
                  MULT3(I,J)=MULT1(I,J)+MULT2(I,J)
                END DO
            END DO

              if(mod(t,86400).eq.0.and.t.lt.86400*10)then
              DO J=1,YGRIDS
                DO I=1,XGRIDS
                  write(77,10)dpprimedmrec(i,j) 
                  write(78,10)drvprimedmrec(i,j)
                end do
              end do
            end if

            if(t.eq.0)then
      OPEN(39,FILE=FPATH//'Fullf')
      DO J=1,YGRIDS
         DO I=1,XGRIDS
            WRITE(39,10)rprimerec(I,J)
         END DO
      END DO
      CLOSE(39)            
      end if

C  Fourier transform of these nonlinear terms

            DO J=1,YGRIDS
               DO I=0,LIM-1
                  MULTFFT3S(I+1,J)=0.
                  MULTFFT3C(I+1,J)=0.
               END DO
            END DO

      do m=0,lim-1
         do ii=1,xgrids
            do jj=1,ygrids
               if(m.eq.0)then
                  bb=MULT3(ii,jj)*cos(m*pi*ii*2/xgrids)/xgrids
                  MULTFFT3C(m+1,jj)=MULTFFT3C(m+1,jj)+bb
               else
                  bb=MULT3(ii,jj)*cos(m*pi*ii*2/xgrids)*2/xgrids
                  MULTFFT3C(m+1,jj)=MULTFFT3C(m+1,jj)+bb
                  aa=MULT3(ii,jj)*sin(m*pi*ii*2/xgrids)*2/xgrids
                  MULTFFT3S(m+1,jj)=MULTFFT3S(m+1,jj)+aa       
               end if
            end do
         end do
      end do

C  Getting the spherical coefficients of the nonlinears by Gaussian quadrature

      DO II=0,LIM-1
         DO JJ=ABS(II),LIM-1
            DO KK=1,YGRIDS
               IF(KK.EQ.1) THEN
                  AA=0.
                  BB=0.
               END IF
               if(ii.lt.0)then
                 ca=1.
                 cb=1.
                 cc=jj-abs(ii)
                 if(cc.gt.1)then
                 do i=1,cc
                   ca=ca*i
                 end do
                 end if
                 cc=jj+abs(ii)
                 if(cc.gt.1)then
                 do i=1,cc
                   cb=cb*i
                 end do
                 end if
                 br=leg(abs(ii),jj,kk)*gwt(kk)*((-1)**ii)*ca/cb
               else
                 BR=LEG(ABS(II),JJ,KK)*GWT(KK)
               end if                                        
               AA=AA+BR*MULTFFT3S(II+1,KK)   
               BB=BB+BR*MULTFFT3C(II+1,KK)
            END DO
            MULTLEGFFT3S(II+1,JJ+1)=AA
            MULTLEGFFT3C(II+1,JJ+1)=BB
         END DO
      END DO

C
C      
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Everything is now in spectral space

      DO M=0,LIM-1
        DO N=ABS(M),LIM-1
            BB=MULTLEGFFT3S(M+1,N+1)
            CC=RPRIMELEGFFTS(M+1,N+1)
            CA=ALPH*RVPRIMELEGFFTS(m+1,N+1)*-1
            CB=-1*alp*DRVPRIMELEGFFT2S(m+1,N+1)
            CE=DRVPRIMELEGFFTS(m+1,N+1)
          if(t.eq.0)then
          DRVPRIMEDTLEGFFTS(M+1,N+1)=BB+CA+CB+CC
          else
          DRVPRIMEDTLEGFFTS(M+1,N+1)=BB+CA+CB+CC
          end if
          if(mod(t,10800).eq.0)then
         ar=real(m)
         br=real(n)
            WRITE(54,10)dRVprimedtlegffts(m+1,n+1),
     1           RVprimelegffts(m+1,n+1),ar,br,cc,bb,ca,cb,ce,cf,t
          end if
            BB=MULTLEGFFT3C(M+1,N+1)
            CC=RPRIMELEGFFTC(M+1,N+1)
            CA=ALPH*RVPRIMELEGFFTC(m+1,N+1)*-1
            CB=-1*alp*DRVPRIMELEGFFT2C(m+1,N+1)
            CE=DRVPRIMELEGFFTC(m+1,N+1)
          if(t.eq.0)then
          DRVPRIMEDTLEGFFTC(M+1,N+1)=BB+CA+CB+CC
          else
          DRVPRIMEDTLEGFFTC(M+1,N+1)=BB+CA+CB+CC
          end if          
          if(mod(t,10800).eq.0)then
            ar=real(m)
            br=real(n)
          WRITE(53,10)dRVprimedtlegfftC(m+1,n+1),
     1           RVprimelegfftC(m+1,n+1),ar,br,cc,bb,ca,cb,ce,cf,t
          end if
        END DO
      END DO

C  Updating FILT=0, means no filter--is needed however to avoid leapfrog
C  related instabilities      

      FILT=1
      IF(FILT.EQ.0)THEN
      DO M=0,LIM-1
        DO N=ABS(M),LIM-1
          if(t.eq.0)then
            HOLDS(M+1,N+1)=RVPRIMELEGFFTS(M+1,N+1)+
     1           DT*DRVPRIMEDTLEGFFTS(M+1,N+1)
            HOLDC(M+1,N+1)=RVPRIMELEGFFTC(M+1,N+1)+
     1           DT*DRVPRIMEDTLEGFFTC(M+1,N+1)
            else
            HOLDS(M+1,N+1)=RVPRIMELEGFFT2S(M+1,N+1)+
     1             2*DT*DRVPRIMEDTLEGFFTS(M+1,N+1)
            HOLDC(M+1,N+1)=RVPRIMELEGFFT2C(M+1,N+1)+
     1           2*DT*DRVPRIMEDTLEGFFTC(M+1,N+1)              
          END IF
            rvprimelegfft2s(m+1,n+1)=rvprimelegffts(m+1,n+1)
            RVPRIMELEGFFTS(M+1,N+1)=HOLDS(M+1,N+1)
            rvprimelegfft2c(m+1,n+1)=rvprimelegfftc(m+1,n+1)
            RVPRIMELEGFFTC(M+1,N+1)=HOLDC(M+1,N+1)    
        END DO  
      END DO

      ELSE

C    If filtering use the below
      
      DO M=0,LIM-1
        DO N=ABS(M),LIM-1
          if(t.eq.0)then
            HOLDS(M+1,N+1)=RVPRIMELEGFFTS(M+1,N+1)+
     1           DT*DRVPRIMEDTLEGFFTS(M+1,N+1)
            HOLDC(M+1,N+1)=RVPRIMELEGFFTC(M+1,N+1)+
     1           DT*DRVPRIMEDTLEGFFTC(M+1,N+1)
          else
            HOLDS(M+1,N+1)=AVGRVPRIMELEGFFT2S(M+1,N+1)+
     1           2*DT*DRVPRIMEDTLEGFFTS(M+1,N+1)
            HOLDC(M+1,N+1)=AVGRVPRIMELEGFFT2C(M+1,N+1)+
     1           2*DT*DRVPRIMEDTLEGFFTC(M+1,N+1)
          end if

C  Robert's Filter

          IF(T.GT.0)THEN
            AR=0.1*(HOLDS(M+1,N+1)-2*RVPRIMELEGFFTS(M+1,N+1)+
     1           AVGRVPRIMELEGFFT2S(M+1,N+1))
            AVGRVPRIMELEGFFT2S(M+1,N+1)=RVPRIMELEGFFTS(M+1,N+1)+AR
            RVPRIMELEGFFT2S(M+1,N+1)=RVPRIMELEGFFTS(M+1,N+1)
            RVPRIMELEGFFTS(M+1,N+1)=HOLDS(M+1,N+1)
            BR=0.1*(HOLDC(M+1,N+1)-2*RVPRIMELEGFFTC(M+1,N+1)+
     1           AVGRVPRIMELEGFFT2C(M+1,N+1))
            AVGRVPRIMELEGFFT2C(M+1,N+1)=RVPRIMELEGFFTC(M+1,N+1)+BR
            RVPRIMELEGFFT2C(M+1,N+1)=RVPRIMELEGFFTC(M+1,N+1)
            RVPRIMELEGFFTC(M+1,N+1)=HOLDC(M+1,N+1)
          ELSE
            AR=0.1*(HOLDS(M+1,N+1)-2*RVPRIMELEGFFTS(M+1,N+1)+
     1           RVPRIMELEGFFT2S(M+1,N+1))
            AVGRVPRIMELEGFFT2S(M+1,N+1)=RVPRIMELEGFFTS(M+1,N+1)+AR
            RVPRIMELEGFFT2S(M+1,N+1)=RVPRIMELEGFFTS(M+1,N+1)
            BR=0.1*(HOLDC(M+1,N+1)-2*RVPRIMELEGFFTC(M+1,N+1)+
     1           AVGRVPRIMELEGFFT2C(M+1,N+1))
            AVGRVPRIMELEGFFT2C(M+1,N+1)=RVPRIMELEGFFTC(M+1,N+1)+BR
            RVPRIMELEGFFT2C(M+1,N+1)=RVPRIMELEGFFTC(M+1,N+1)            
          END IF            

        END DO
      END DO
      END IF
      end if
      
C  Saving the end of the day

      AR=MOD(T,86400*5)
      IF(AR.EQ.0.and.t.gt.0)THEN
        write(*,*)t,'time'
        WCNT=WCNT+1
        WRITE (OUTCH, '(I3)'), WCNT

        DO M=0,LIM-1
          DO N=ABS(M),LIM-1
            IF(N.GT.0)THEN
              AR=-1*RADI*RADI/N/(N+1)
              PPRIMELEGFFTS(M+1,N+1)=avgRVPRIMELEGFFT2S(M+1,N+1)*AR
              PPRIMELEGFFTC(M+1,N+1)=avgRVPRIMELEGFFT2C(M+1,N+1)*AR
            else
              PPRIMELEGFFTS(M+1,N+1)=0.
              PPRIMELEGFFTC(M+1,N+1)=0.
            end if
          END DO
        END DO

C  Performing the reverse Legendre transform on the current prime fields

        do j=1,ygrids
          do i=0,lim-1
            pprimerec1s(i+1,j)=0.
            pprimerec1c(i+1,j)=0.
            rvprimerec1s(i+1,j)=0.
            rvprimerec1c(i+1,j)=0.
          end do
        end do

        DO KK=1,YGRIDS
          DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
              AA=PPRIMELEGFFTS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              PPRIMEREC1S(II+1,KK)=PPRIMEREC1S(II+1,KK)+AA
              AA=PPRIMELEGFFTC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              PPRIMEREC1C(II+1,KK)=PPRIMEREC1C(II+1,KK)+AA              
              AA=avgRVPRIMELEGFFT2S(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              RVPRIMEREC1S(II+1,KK)=RVPRIMEREC1S(II+1,KK)+AA
              AA=avgRVPRIMELEGFFT2C(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              RVPRIMEREC1C(II+1,KK)=RVPRIMEREC1C(II+1,KK)+AA
            END DO
          END DO
        END DO

C       Performing the reverse Fourier transform

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            PPRIMEREC(I,J)=0.
            RVPRIMEREC(I,J)=0.
         END DO
      END DO

      do m=1,lim-1
         do j=1,ygrids
            do i=1,xgrids
               AA=PPRIMEREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=PPRIMEREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               PPRIMEREC(I,J)=PPRIMEREC(I,J)+AA+BB
               AA=RVPRIMEREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=RVPRIMEREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               RVPRIMEREC(I,J)=RVPRIMEREC(I,J)+AA+BB
            END DO
         END DO
      END DO

      OPEN(50,FILE=FPATH//'Pprimerec'//OUTCH)
      DO J=1,YGRIDS
        DO I=1,XGRIDS
          WRITE(50,10)PPRIMEREC(I,J)
        END DO
      END DO
      CLOSE(50)
      
      OPEN(51,FILE=FPATH//'Rvprimerec'//OUTCH)
      DO J=1,YGRIDS
        DO I=1,XGRIDS
          WRITE(51,10)RVPRIMEREC(I,J)
        END DO
      END DO
      CLOSE(51)
        
      END IF

CC  Saving out the first 10 days

      AR=MOD(T,86400)
      IF(AR.EQ.0.AND.T.GT.0.AND.T.LE.86400*10)THEN
        write(*,*)t,'time'
        DCNT=DCNT+1
        WRITE (OUTCH, '(I3)'), DCNT

        DO M=0,LIM-1
        DO N=ABS(M),LIM-1
            IF(N.GT.0)THEN
              AR=-1*RADI*RADI/N/(N+1)
              PPRIMELEGFFTS(M+1,N+1)=RVPRIMELEGFFTS(M+1,N+1)*AR
              PPRIMELEGFFTC(M+1,N+1)=RVPRIMELEGFFTC(M+1,N+1)*AR
            else
              PPRIMELEGFFTS(M+1,N+1)=0.
              PPRIMELEGFFTC(M+1,N+1)=0.
            end if            
          END DO
        END DO

C  Performing the reverse Legendre transform on the current prime fields

        do j=1,ygrids
          do i=0,lim-1
            pprimerec1s(i+1,j)=0.
            pprimerec1c(i+1,j)=0.
            rvprimerec1s(i+1,j)=0.
            rvprimerec1c(i+1,j)=0.
          end do
        end do

        DO KK=1,YGRIDS
          DO II=0,LIM-1
            DO JJ=ABS(II),LIM-1
              AA=PPRIMELEGFFTS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              PPRIMEREC1S(II+1,KK)=PPRIMEREC1S(II+1,KK)+AA
              AA=PPRIMELEGFFTC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              PPRIMEREC1C(II+1,KK)=PPRIMEREC1C(II+1,KK)+AA              
              AA=RVPRIMELEGFFTS(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              RVPRIMEREC1S(II+1,KK)=RVPRIMEREC1S(II+1,KK)+AA
              AA=RVPRIMELEGFFTC(II+1,JJ+1)*LEG(ABS(II),JJ,KK)
              RVPRIMEREC1C(II+1,KK)=RVPRIMEREC1C(II+1,KK)+AA
            END DO
          END DO
        END DO

C       Performing the reverse Fourier transform

      DO J=1,YGRIDS
         DO I=1,XGRIDS
            PPRIMEREC(I,J)=0.
            RVPRIMEREC(I,J)=0.
         END DO
      END DO

      do m=1,lim-1
        do j=1,ygrids
            do i=1,xgrids
               AA=PPRIMEREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=PPRIMEREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               PPRIMEREC(I,J)=PPRIMEREC(I,J)+AA+BB
               AA=RVPRIMEREC1S(m+1,j)*SIN(M*2*PI*I/XGRIDS)
               BB=RVPRIMEREC1C(M+1,J)*COS(M*2*PI*I/XGRIDS)
               RVPRIMEREC(I,J)=RVPRIMEREC(I,J)+AA+BB
            END DO
         END DO
      END DO

      OPEN(80,FILE=FPATH//'Pprime'//OUTCH)
      DO J=1,YGRIDS
        DO I=1,XGRIDS
          WRITE(80,10)PPRIMEREC(I,J)
        END DO
      END DO
      CLOSE(80)
      
      OPEN(81,FILE=FPATH//'Rvprime'//OUTCH)
      DO J=1,YGRIDS
        DO I=1,XGRIDS
          WRITE(81,10)RVPRIMEREC(I,J)
        END DO
      END DO
      CLOSE(81)
        
      END IF      

      
      A=MOD(T,3600)
      IF(A.EQ.0)THEN
        WRITE(*,*)T
      END IF
    
      END DO

      close(54)
      close(53)
      close(77)
      close(78)
      CLOSE(79)
      
      return
      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        SUBROUTINE MLPMN2(M,N,X,PM,PD)
C
C       ==========================================================
C       Purpose: This program computes the NORMALIZED associated Legendre 
C                functions Pmn(x) and their derivatives Pmn'(x) 
C                using subroutine LPMN
C       Input :  x --- Argument of Pmn(x)
C                m --- Order of Pmn(x),  m = 0,1,2,...,n
C                n --- Degree of Pmn(x), n = 0,1,2,...,N
C       Output:  PM(m,n) --- Pmn(x)
C                PD(m,n) --- Pmn'(x)
C       Example: x = 0.50
C          Pmn(x):
C          m\n        1            2            3            4
C         --------------------------------------------------------
C           0      .500000     -.125000     -.437500     -.289063
C           1     -.866025    -1.299038     -.324760     1.353165
C           2      .000000     2.250000     5.625000     4.218750
C           3      .000000      .000000    -9.742786   -34.099750
C           4      .000000      .000000      .000000    59.062500
C
C          Pmn'(x):
C          m\n        1            2            3            4
C         --------------------------------------------------------
C           0     1.000000     1.500000      .375000    -1.562500
C           1      .577350    -1.732051    -6.278684    -5.773503
C           2      .000000    -3.000000     3.750000    33.750000
C           3      .000000      .000000    19.485572      .000000
C           4      .000000      .000000      .000000  -157.500000
C       ==========================================================
C
        IMPLICIT DOUBLE PRECISION (P,X)
        DIMENSION PM(0:100,0:100),PD(0:100,0:100)
C        WRITE(*,*)'  Please enter m, n and x'
C        READ(*,*) M,N,X
C        WRITE(*,*)
C        WRITE(*,*)'  m     n      x          Pmn(x)         Pmn''(x)'
C        WRITE(*,*)' ---------------------------------------------------'
        CALL LPMN2(100,M,N,X,PM,PD)
C        DO 15 J=0,N
C           WRITE(*,10)j,N,X,PM(J,N),PD(J,N)
C15      CONTINUE
C10      FORMAT(1X,I3,3X,I3,3X,F5.1,2E17.8)
        END


        SUBROUTINE LPMN2(MM,M,N,X,PM,PD)
C
C       =====================================================
C       Purpose: Compute the associated Legendre functions 
C                Pmn(x) and their derivatives Pmn'(x)
C       Input :  x  --- Argument of Pmn(x)
C                m  --- Order of Pmn(x),  m = 0,1,2,...,n
C                n  --- Degree of Pmn(x), n = 0,1,2,...,N
C                mm --- Physical dimension of PM and PD
C       Output:  PM(m,n) --- Pmn(x)
C                PD(m,n) --- Pmn'(x)
C       =====================================================
C
        IMPLICIT DOUBLE PRECISION (P,X)
        DIMENSION PM(0:MM,0:N),PD(0:MM,0:N)
        DOUBLE PRECISION AR,BR,NRM,PPA,PPB
        DIMENSION PN(0:MM,0:N)
        INTEGER K,LT,LS

        DO 10 I=0,N
        DO 10 J=0,M
           PM(J,I)=0.0D0
10         PD(J,I)=0.0D0
        PM(0,0)=1.0D0
        IF (DABS(X).EQ.1.0D0) THEN
           DO 15 I=1,N
              PM(0,I)=X**I
15            PD(0,I)=0.5D0*I*(I+1.0D0)*X**(I+1)
           DO 20 J=1,N
           DO 20 I=1,M
              IF (I.EQ.1) THEN
                 PD(I,J)=1.0D+300
              ELSE IF (I.EQ.2) THEN
                 PD(I,J)=-0.25D0*(J+2)*(J+1)*J*(J-1)*X**(J+1)
              ENDIF
20         CONTINUE
           RETURN
        ENDIF
        LS=1
        IF (DABS(X).GT.1.0D0) LS=-1
        XQ=DSQRT(LS*(1.0D0-X*X))
        XS=LS*(1.0D0-X*X)
        DO 30 I=1,M
30         PM(I,I)=-LS*(2.0D0*I-1.0D0)*XQ*PM(I-1,I-1)
        DO 35 I=0,M
35         PM(I,I+1)=(2.0D0*I+1.0D0)*X*PM(I,I)
        DO 40 I=0,M
        DO 40 J=I+2,N
           PM(I,J)=((2.0D0*J-1.0D0)*X*PM(I,J-1)-
     &             (I+J-1.0D0)*PM(I,J-2))/(J-I)
40      CONTINUE

        DO I=0,M
        DO J=I,N
           LT=J-I
           PPA=1.0D0
           IF(LT.GT.1)THEN
              DO K=2,LT
                 PPA=PPA*K
              END DO
           END IF
           LS=J+I
           PPB=1.0D0
           IF(LS.GT.1)THEN
              DO K=2,LS
                 PPB=PPB*K
              END DO
           END IF
           AR=(PPA/PPB)**(0.5)
           BR=((2*REAL(J)+1)/2)**(0.5)
           NRM=AR*BR   !*(-1)**I
           PM(I,J)=PM(I,J)*NRM
c           WRITE(*,*)NRM,AR,BR,PM(I,J)
        END DO
        END DO


        PD(0,0)=0.0D0
        DO 45 J=1,N
45         PD(0,J)=LS*J*(PM(0,J-1)-X*PM(0,J))/XS
        DO 50 I=1,M
        DO 50 J=I,N
           PD(I,J)=LS*I*X*PM(I,J)/XS+(J+I)
     &             *(J-I+1.0D0)/XQ*PM(I-1,J)
50      CONTINUE
        RETURN
        END
