%%%%%%%%%%%%%%%%%%%%
%%%  Matlab script to drive 'backtime2.f'
%%%  Following Branstator, Krishnamurthi and Haltiner and Williams

%%%  Loading the mean 2-dimensional 200mb streamfunction

ld=2;
if ld==1
  
loaddods('http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.MONTHLY/.Intrinsic/.PressureLevel/.psi/T/%28Aug-Oct%201949-2006%29VALUES/P/%28200%29%28200%29RANGEEDGES%5BT%5Daverage/dods')

loaddods('http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.MONTHLY/.Intrinsic/.PressureLevel/.relv/T/%28Aug-Oct%201949-2006%29VALUES/P/%28200%29%28200%29RANGEEDGES%5BT%5Daverage/dods')

x=transpose(X);
y=Y;
Y=y*ones(1,144);
X=ones(73,1)*x;
     ylat=Y*pi/180;
     xlong=X*pi/180;

%%  Getting the zeros and weights of the nth order Legendre polynomial
%%  That is, the Gaussian latitudes (the zeros), and the corresonding weights

%%  Using grid generated with lgwt.m

glat=72;
[Zs,Ws]=lgwt(glat,-1,1);
glat2=length(Zs);
hh=Y*pi/180;
Ynorm=sin(Y*pi/180);

%%  Getting new grid of Gaussian latitudes and equally spaced longitudes 
%%  for interpolation.

Ygaus=Zs*ones(1,144);
Xgaus=ones(glat2,1)*x;

else
  
load jas300mb_clim49-04_ncepncar_relv
load jas300mb_clim49-04_ncepncar_psi

x=0:2.5:357.5;
Y=90:-2.5:-90;
y=Y';
Y=y*ones(1,144);
X=ones(73,1)*x;
     ylat=Y*pi/180;
     xlong=X*pi/180;
glat=72;
[Zs,Ws]=lgwt(glat,-1,1);
glat2=length(Zs);
hh=Y*pi/180;
Ynorm=sin(Y*pi/180);
Ygaus=Zs*ones(1,144);
Xgaus=ones(glat2,1)*x;

end 

psigaus=interp2(X,Ynorm,psi,Xgaus,Ygaus,'spline');
relvgaus=interp2(X,Ynorm,relv,Xgaus,Ygaus,'spline');

Pbar=psigaus;
RVbar=relvgaus;
 
%%  Filtering 

filt=1;
cut=9;

if filt==1

xgrids=144;
PbarfftC=zeros(glat2,2*cut+1);
PbarfftS=zeros(glat2,2*cut+1);

for m=-1*cut+1:cut-1
for j=1:144
for i=1:glat2
  aa=Pbar(i,j)*sin(m*pi*j*2/xgrids)*2/xgrids;
if m==0
bb=Pbar(i,j)*cos(m*pi*j*2/xgrids)/xgrids;
else
bb=Pbar(i,j)*cos(m*pi*j*2/xgrids)*2/xgrids;
end
PbarfftC(i,m+cut)=PbarfftC(i,m+cut)+bb;
PbarfftS(i,m+cut)=PbarfftS(i,m+cut)+aa;
end
end
end

Pbartry=zeros(glat2,144);
for m=0:cut
for i=1:glat2
for j=1:144
aa=PbarfftS(i,m+cut)*sin(m*2*pi*j/xgrids);
bb=PbarfftC(i,m+cut)*cos(m*2*pi*j/xgrids);
Pbartry(i,j)=Pbartry(i,j)+aa+bb;
end
end
end

elseif filt==2

for i=1:glat2
Pbarfft(i,:)=fft(Pbar(i,:));
end

XX=0:143;
n=length(x);
for j=1:glat2
for i=1:3        %% i=1:144 would use all Fourier modes and be exact
 tmp(i,:)=real(Pbarfft(j,i)*exp((-1)^0.5*(i-1)*XX*2*pi/n))/n;
end
Pbartry(j,:)=sum(tmp);
end
end

if filt==1 | filt==2
corr2(Pbar,Pbartry)
Pbar=Pbartry;
end

%%  Below is solid body rotation
%%  i.e. u=C*cos(lat); U=u*cos(lat)/a and U=(-1/rad/rad)*cos2(lat)*dPsi/dmu

soldbdy=0;
if soldbdy==1
  Pbar=-15.4*rad*Zs;
  Pbar=Pbar*ones(1,144);
end
  
rad=6.37e6;

Pbar2=reshape(Pbar,144*glat,1);
RVbar2=reshape(RVbar,144*glat,1);
GL=[Zs Ws];

save startPbar Pbar2 -ascii
save startRVbar RVbar2 -ascii
save glatswts GL -ascii

unix('rm a.out');
unix('g77 -fno-automatic backtime2.f');
unix('a.out')

load Pbarrec
Pbarrec=transpose(reshape(Pbarrec,144,72));

figure(1)
 subplot(2,1,1);surf(Xgaus,Ygaus,Pbar);view(2);shading flat;colorbar('vert')
 subplot(2,1,2);surf(Xgaus,Ygaus,Pbarrec);view(2);shading flat;colorbar('vert')

Ygaus2=asin(Ygaus)*180/pi;
for i=1:5
  figure(40+i)
  for j=1:2
  eval(sprintf('load Pprimerec%d',2*i-2+j+100));
  eval(sprintf('Pprc=transpose(reshape(Pprimerec%d,144,72));',2*i-2+j+100));
  subplot(2,2,2*j-1);
m_proj('equidistant','lon',[0 357.5],'lat',[-88 88]);
m_pcolor(Xgaus,Ygaus2,Pprc); shading flat
m_coast('linewidth',1,'color','k');
m_grid
colorbar('vert')  
  ii=num2str(i*10-10+5*j);tz=['Psi Anomaly Day ', ii]; title(tz)
  end
   
for j=1:2
  eval(sprintf('load Rvprimerec%d',2*i-2+j+100));
  eval(sprintf('Rvprc=transpose(reshape(Rvprimerec%d,144,72));',2*i-2+j+100));
  subplot(2,2,2*j);
m_proj('equidistant','lon',[0 357.5],'lat',[-88 88]);
m_pcolor(Xgaus,Ygaus2,Rvprc);shading flat
m_coast('linewidth',1,'color','k');
m_grid
colorbar('vert')  
  ii=num2str(i*10-10+5*j);tz=['Relv Anomaly Day ',ii];title(tz)
end
end

Ygaus2=asin(Ygaus)*180/pi;
figure(25)
subplot(2,1,1)
m_proj('equidistant','lon',[0 357.5],'lat',[-88 88]);
m_contourf(Xgaus,Ygaus2,Pprc,[15]);
m_coast('linewidth',1,'color','k');
m_grid
colorbar('vert')
title('JAS 300mb 1949-2004 0-8 Filter; 160-280E, 0-10N Forcing; T24')
subplot(2,1,2)
m_proj('equidistant','lon',[0 357.5],'lat',[-88 88]);
m_contourf(Xgaus,Ygaus2,Rvprc,[15]);
m_coast('linewidth',1,'color','k');
m_grid
colorbar('vert')
  
%%  These next 2 plot the first 10 days (days 0-9).  
  
for i=1:5
  figure(30+i)
  for j=1:2
  eval(sprintf('load Pprime%d',2*i-2+j+100));
  eval(sprintf('Pprc=transpose(reshape(Pprime%d,144,72));',2*i-2+j+100));
  subplot(2,2,2*j-1);
m_proj('equidistant','lon',[0 357.5],'lat',[-88 88]);
m_pcolor(Xgaus,Ygaus2,Pprc);shading flat
m_coast('linewidth',1,'color','k');
m_grid
colorbar('vert')  
  ii=num2str(i*2-2+j);tz=['Psi Anomaly Day ', ii];title(tz)
  end
   
for j=1:2
  eval(sprintf('load Rvprime%d',2*i-2+j+100));
  eval(sprintf('Rvprc=transpose(reshape(Rvprime%d,144,72));',2*i-2+j+100));
  subplot(2,2,2*j);
m_proj('equidistant','lon',[0 357.5],'lat',[-88 88]);
m_pcolor(Xgaus,Ygaus2,Rvprc);shading flat
m_coast('linewidth',1,'color','k');
m_grid
colorbar('vert')  
  ii=num2str(i*2-2+j);tz=['Relv Anomaly Day ',ii];title(tz)
  end
end

for i=1:9
  if rem(i,3)==1
    figure(20+i)
  end
  um=rem(i,3);
  if um==0
    um=3;
  end
  subplot(3,1,um)
  eval(sprintf('load Rvprime%d',i+100));
  eval(sprintf('Rvprc=transpose(reshape(Rvprime%d,144,72));',i+100));
  m_proj('equidistant','lon',[0 357.5],'lat',[-20 60]);
  [cs,h]=m_contourf(Xgaus,Ygaus2,Rvprc,[-1e-5 -5e-6 -2e-6:1e-6:2e-6 1e-5 5e-6],'k');
  caxis([-1e-5 1e-5]);
  clear uu
            for j=1:length(h)
            uu(j)=get(h(j),'UserData');
            end
            zz=[];gz=[];
            for j=1:length(uu)
            if uu(j)==0
            zz=[zz;j];
            elseif uu(j)>0
            gz=[gz;j];
            end
            end
            if isempty(zz)
            set(h(1:min(gz)-1,1),'LineStyle','--')
            else
            set(h(1:min(zz)-1,1),'LineStyle','--')
            set(h(min(zz):max(zz),1),'LineWidth',2)
            end
  m_coast('linewidth',1,'color','k');
  m_grid
  colorbar('vert')  
  ii=num2str(i);tz=['\zeta Anomaly Day ', ii];title(tz)
end
   
